﻿// ****************************************************************************************
// Copyright (C) 2010, Jorn Skaarud Karlsen 
// All rights reserved. 
//
// This file is part of VoxelloScope (http://www.voxelloscope.org/).
//
// VoxelloScope is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VoxelloScope is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VoxelloScope.  If not, see <http://www.gnu.org/licenses/>.
//***************************************************************************************

#define RESULT_SIZE 2046

//--------------------------------------------------------------------------------------
// Textures and frame values..
//--------------------------------------------------------------------------------------

RWStructuredBuffer<uint> LoadBuffer;
RWStructuredBuffer<uint> FreeBuffer;
RWStructuredBuffer<uint> BricksCache;

cbuffer Constants
{
   uint FrameCounter;
   uint BrickCount;
   uint PreloadBrickCount;
}

[numthreads(256, 1, 1)]
void ProcessCache( uint3 DTid : SV_DispatchThreadID )
{
    uint lookupIndex = DTid.x;
	if (lookupIndex < PreloadBrickCount || // These bricks are always in cache for fast LODing..
	    lookupIndex >= BrickCount)         // Only perform caching for valid indices..
	{
		return;
	}

    uint cacheValue = BricksCache[lookupIndex];

	if (cacheValue == 0) // Brick needs to be loaded..
	{
		uint oldCounterValue;
		InterlockedAdd(LoadBuffer[0], 1, oldCounterValue);
		if (oldCounterValue < RESULT_SIZE)
		{
           LoadBuffer[oldCounterValue + 1] = lookupIndex;
		}
	}
	else if (cacheValue != 0xFFFFFFFF && (FrameCounter - cacheValue) > 20) // Brick was not used in the active frame and can be replaced..
	{
		uint oldCounterValue;
		InterlockedAdd(FreeBuffer[0], 1, oldCounterValue);
		if (oldCounterValue < RESULT_SIZE)
		{
			FreeBuffer[oldCounterValue + 1] = lookupIndex;
		}
	}
}
